﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ThreadPoolSynchronization
{
    public class ThreadPoolSynchronizer : SynchronizationContext, IDisposable
    {
        Queue<WorkItem> workItemQueue;
        WorkerThread[] workerThreads;
        Semaphore itemAdded;

        public ThreadPoolSynchronizer(int poolSize)
        {
            if (poolSize <= 0)
                throw new InvalidOperationException("Pool size cannot be zero");

            workItemQueue = new Queue<WorkItem>();

            workerThreads = new WorkerThread[poolSize];
            for (int index = 0; index < poolSize; index++)
                workerThreads[index] = new WorkerThread(index + 1, this);
        }

        public void Close()
        {
            foreach (WorkerThread thread in workerThreads)
                thread.Abort();
        }

        public void Abort()
        {
            foreach (WorkerThread thread in workerThreads)
                thread.Abort();
        }

        public void Dispose()
        {
            this.Close();
        }

        public override void Post(SendOrPostCallback method, Object state)
        {
            WorkItem workItem = new WorkItem(method, state);
            QueueWorkItem(workItem);
        }

        public override void Send(SendOrPostCallback method, Object state)
        {
            if (SynchronizationContext.Current == this)
            {
                method(state);
                return;
            }
            WorkItem workItem = new WorkItem(method, state);
            QueueWorkItem(workItem);
            workItem.AsyncWaitHandle.WaitOne();
        }

        protected Semaphore ItemAdded
        {
            get
            {
                if (itemAdded == null)
                    itemAdded = new Semaphore(0, Int32.MaxValue);

                return itemAdded;
            }
            set
            {
                itemAdded = value;
            }
        }

        virtual internal void QueueWorkItem(WorkItem workItem)
        {
            lock (workItemQueue)
            {
                workItemQueue.Enqueue(workItem);
                ItemAdded.Release();
            }
        }

        protected virtual bool QueueEmpty
        {
            get
            {
                lock (workItemQueue)
                {
                    if (workItemQueue.Count > 0)
                    {
                        return false;
                    }
                    return true;
                }
            }
        }
        internal virtual WorkItem GetNext()
        {
            ItemAdded.WaitOne(1000);
            lock (workItemQueue)
            {
                if (workItemQueue.Count == 0)
                {
                    return null;
                }
                return workItemQueue.Dequeue();
            }
        }
 
    }

}
